/**     Fractional Integration and Brownian Motion Programs **/

/****************************************************************************

**                     FBM (Fractional Brownian Motion)

**                     ********************************

**                    Format: {Wq,DeltaW} = FBM(N,q)

**                    Inputs: q:  -.5<0<.5 degree of integration

**                            N:  no. of points at which FMB evaluated

**                                FBM is generated at points r = 1/N,2/N...1

**                                N must be quite high to get accuracy

**                    Outputs:

**                             Wq:  N by 1 vector of standard FBM

**                             Wq[i] approximates Wq(r)for r=1/N where

**                                Wq(r) =1/gamma(q+1)*int_0^r (r-x)^q dW(x)

**                                where W=BM(1) is Standard Brownian Motion

**                             DeltaW N+1 by 1 vector of iid N(0,1/N) variables

**                                DeltaW = (W(i/N)-W((i-1)/N)

**

**                   Uses:  int_0^1 Wq(r)dr = 1/N sumc(Wq)

**                          int_0^1 Wq(r)dW = 1/N Wq'DeltaW[2:N+1]

**                              (note must use forward differentail)

**                          xy(seqa(1/N,1/N,N),Wq) plots the FBM

**

****************************************************************************/





proc(2) =  FBM(N,q);

local eps, Wq, nn, gam, rminusxq, DeltaW;

gam = gamma(q+1);

eps = rndn(N+1,1);

Wq = zeros(N,1);



nn = 1;

do until nn>N;           /* r =nn/N */

    if nn == 1;   /*to keep computer from estimating 0 by a neg. number*/

    rminusxq = 0;

    elseif nn>1;

    rminusxq =((nn/N-seqa(1/N,1/N,nn-1)).^q)|0;

    endif;

  Wq[nn] = 1/gam*rminusxq'eps[1:nn]/sqrt(N);

  nn = nn+1;

endo;

DeltaW = eps/sqrt(N);

retp(Wq,DeltaW);

endp;

/***************************************************************************

**                  FBM2.g (Fractional Brownian Motion 2)

**                  *************************************

**            Format:   {lim1,lim2,lim3} = FBM2(M,N,q)

**            Inputs:   M: Number of simulations

**                      N: Number of points used to approximate FBMs in each

**                         simulation

**                      q: Degree of integration of the FBM

**            Outputs:  lim1 = M by 1 vector of draws from approximations to

**                          1/int_0^1 _Wq(r)^2dr

**                      lim2 = M by 1 vector of draws from approximations to

**                         int _WqdW/int _Wq^2

**                      lim3 = M by 1 vector of draws from approximations to

**                         int _WqdW* /int Wq^2, where W* is indep of W

**            for Greater Detail:  refer to FBM.g

***************************************************************************/



proc(3) = FBM2(M,N,q);

local lim1,lim2,lim3,Wq,DeltaW,Wqdr,_Wq,_Wq_Wq,_WqdW,_WqdW2,mm;

lim1 = zeros(M,1);

lim2 = zeros(M,1);

lim3 = zeros(M,1);

mm = 1;

do until mm>M;

        {Wq,DeltaW} = FBM(N,q);

        Wqdr        = 1/N*sumc(Wq);

        _Wq         = Wq-Wqdr;

        _Wq_Wq      = 1/N*_Wq'_Wq;

        _WqdW       = _Wq'DeltaW[2:N+1,1];

        _WqdW2      = _Wq'rndn(N,1)/sqrt(N);

        lim1[mm]    = 1/_Wq_Wq;

        lim2[mm]    = _WqdW/_Wq_Wq;

        lim3[mm]    = _WqdW2/_Wq_Wq;

        if mm>20 and N>4000;

           save lim1;

           save lim2;

           save lim3;

           print mm;

        endif;

        mm = mm+1;

        endo;

retp(lim1,lim2,lim3);

endp;

/****************************************************************************

**                                  FBM3.g

**              for use in approximation to finite distribution

**              of hat(beta) in the return Premium Regression.

**                              (Alex Maynard)

**                     ********************************

**             Format:  lim4 = fbm3(lim1,lim2,lim3,settings)

**             Output: lim4:  A draw from asymptotic estimate of finite sample

**                     distribution of return Premium slope estimate using

**                     contribution from both the n^(2d-1) and the n^d term.

**

**             Inputs:  lim1 = M by 1 vector of draws from approximations to

**                          1/int_0^1 _Wq(r)^2dr

**                      lim2 = M by 1 vector of draws from approximations to

**                         int _WqdW/int _Wq^2

**                      lim3 = M by 1 vector of draws from approximations to

**                         int _WqdW* /int Wq^2, where W* is indep of W

**                      settings: 8 by 1 vector containing following info:

**                           (caplam,capsig,omegxx,omegvv,omegxv,k,n,d)';

**                        (refer to paper for defn's of the following)

**                      1  caplam: Lamda_{Delta i,sr}

**                      2  capsig: Sigam_{u,sr}

**                      3  omegxx: Omega_{xi xi}

**                      4  omegvv: Omega_{nu nu}

**                      5  omegxv: Omega_{xi nu}

**                      6  k:      forward contract length

**                      7  n:      length of data set used

**                      8  d:      degree of integrations of interest

**                                differential

**             NOTE: d must be the same as is used to make lim1, lim2, and lim3

**                   otherwize results are meaningless

**             calls:

****************************************************************************/





proc(1) = fbm3(lim1,lim2,lim3,settings);

local c1,c2,c3,lim4,caplam,capsig,omegxx,omegvv,omegxv,k,n,d,sig_si;



caplam  =  settings[1];

capsig  =  settings[2];

omegxx  =  settings[3];

omegvv  =  settings[4];

omegxv  =  settings[5];

k       =  settings[6];

n       =  settings[7];

d       =  settings[8];

sig_si = sqrt(omegvv-omegxv^2/omegxx);

c1 = (caplam+capsig)/omegxx;

c2 = k*omegxv/omegxx;

c3 = k*sig_si/sqrt(omegxx);

lim4 = n^(1-2*d)*c1*lim1+n^(-d)*(c2*lim2+c3*lim3);

retp(lim4);

endp;

/***************************************************************************

**                              lp_reg.g

****************************************************************************

**          Log Periodogram Regression: using first rrr2 values

**

**                        {d, b, Iww, se} = lp_reg(x,rrr,rrr2)

**

**          d = estimated degree of fractional integration

**          b = 2 by 1 vector of slope & intercept coeffs from lp regression

**          Iww = Periodogram using 1st through rrr'th frequency

**          se = standard error of d

**          rrr = 1st - rrr'th frequency for Iww

**          rrr2 = 1st-rrr2'th frequency used in log periodogram regression

****************************************************************************/



proc(4)= lp_reg(x,rrr, rrr2);

local z,y,d,b,se, _output, vnam, m, stb, vc, stderr, sigma, cx, rsq, resid,

            dwstat, Iww, T;

t=rows(x);

Iww = periodo(x,rrr);





z = ln(2*pi*seqa(1,1,rows(Iww))/T);

y = ln(Iww);

z = z[1:minc(rrr2|rrr)];

y = y[1:minc(rrr2|rrr)];



/*print country;*/

"----------";

print;

"Log Periodogram Regression Results";

"----------------------------------";

print;



_output = 0;

b =  OLSQR(y,ones(rows(z),1)~z);



print;

print;

"Estimated order of integration";

"------------------------------";

print;

"   d     standard error  ";

"----------------------- ";

print;

d = -b[2]/2;

se = pi/sqrt(24)/sqrt(rrr2);

print d~se;

print;

print;





retp(d,b,Iww,se);

endp;

/************************************************************************/

















/***************************************************************************

**                              lp_reg2.g

****************************************************************************

**          Log Periodogram Regression: using first rrr2 values

**              (Includes a graph of the fitted regression)

**                        {d, b, Iww,se} = lp_reg2(x,rrr,rrr2,_title,

**                                                _xlabel,_ylabel)

**

**          d = estimated degree of fractional integration

**          b = 2 by 1 vector of slope & intercept coeffs from lp regression

**          Iww = Periodogram using 1st through rrr'th frequency

**          se = standard error of d

**          rrr = 1st - rrr'th frequency for Iww

**          rrr2 = 1st-rrr2'th frequency used in log periodogram regression

**          To cancel the graph: set _title = 0

****************************************************************************/



proc(4)= lp_reg2(x,rrr, rrr2,_title, _xlabel,_ylabel);

local z,y,d,b,se,_output, vnam, m, stb, vc, stderr, sigma, cx, rsq, resid,

            dwstat, Iww, T;

t=rows(x);

Iww = periodo(x,rrr);





z = ln(2*pi*seqa(1,1,rows(Iww))/T);

y = ln(Iww);

z = z[1:minc(rrr2|rrr)];

y = y[1:minc(rrr2|rrr)];



/*print country;*/

"----------";

print;

"Log Periodogram Regression Results";

"----------------------------------";

print;



_output = 0;

b = OLSQR(y,ones(rows(z),1)~z);

print;

print;

"Estimated order of integration";

"------------------------------";

print;

"   d     standard error  ";

"----------------------- ";

print;

d = -b[2]/2;

se = pi/sqrt(24)/sqrt(rrr2);

print d~se;

print;

print;

if _title $== 0;

else;

library pgraph;

title(_title);

xlabel(_xlabel);

ylabel(_ylabel);

_plctrl = {-1, 0};

_pstype = 8;

_psymsiz = 1;

_pnumht = .1;

xy(z,y~(ones(rows(z),1)~z)*b);

endif;

retp(d,b,Iww,se);

endp;

/************************************************************************/







/*************************************************************************

**             Conditional Sum of Squares Estimator: Chung & Baillie

(d,ar,ma,mu,sig) = cssd(y,p,q)

inputs

y:  time series

p:  autoregressive order

q:  moving average order

outputs

d:  fractional parameters

ar: autoregressive parameters

ma: moving average parameters

mu: estimated mean of y

sig: standard deviation of original innovation: after differencing and

    whitening.

*************************************************************************/

/*

proc(5) = cssd(y,p,q);

local d,ar,ma,mu,sig,u,w,e,ss,i,par0,par,f,g,cov,retcode;

@use max like to minimize the function s defined below@

par0 = 1/3*rndn(p+q+3,1);

par0[p+q+3] = abs(par0[p+q+3]);

{par,f,g,cov,retcode} = maxlik(y,1,&s,par0);

d = par[1];

if p >0;

        ar = par[2:1+p];

endif;

if q >0;

        ma = par[P+2:P+1+q];

endif;

mu = par[p+2+q];

sig = par[p+Q+3];

retp(d,ar,ma,mu,sig);

endp;

*/

proc s(y,par);

local ss,u,e,d,ar,ma,mu,sig,w,i,p,q;

d = par[1];

if p >0;

        ar = par[2:1+p];

endif;

if q >0;

        ma = par[P+2:P+1+q];

endif;

mu = par[p+2+q];

sig = par[p+q+3];



u = d_dif2(y,d,0, "_xlabel","_ylabel");  @ARMA processed: d-differenced@

W = u[1+p:rows(u)];

i = 1;

do until i>p;

 w = w -ar[i]*u[1+p-i:rows(u)-i];

i =i+1;

endo;

@***MUST TAKE OUT MA COMPONENT HERE***@

e = W; @original innovation@

ss = -.5*ln(sig^2)+1/(2*sig^2)*e'e;

@****CHECK DEFINITION OF LOG IN GAUSS******@

retp(ss);

endp;



/*************************************************************************

**                       D_DIF2.G

**************************************************************************

**                (fractionally differences the data)

**                (option to graph the differenced data)

**                resid = d_dif2(x,d,_title, _xlabel,_ylabel);

**                x - I(d) series to be differenced

**                set _title =0 to avoid graphing

*************************************************************************/



proc d_dif2(x,d,_title, _xlabel,_ylabel);

local resid, arcoef,jj;



/* Fractionally difference the fp premium and extract the residual*/

resid = zeros(rows(x),1);

arcoef = zeros(rows(x),1);  /*coeffs pi_jj where x_t + sum_{j=1}^inf pi_{t-jj}

                = epsil(t) = I(0) from Brockwell and Davis p465*/

arcoef[1] = -d;

resid[1] = x[1];

jj = 2;

do until jj >rows(x);

    arcoef[jj] = (jj-1-d)/jj*arcoef[jj-1];

    resid[jj] = x[jj]+arcoef[1:jj-1]'rev(x[1:jj-1]);

jj = jj+1;

endo;



/*graph the differenced data*/

if _title $== 0;

else;

begwind;

window(2,1,1);

title("d-Differenced " $+_title );

xlabel(_xlabel);

ylabel("(1-L)^d" $+ _ylabel);

xy(seqa(1,1,rows(resid)),resid);



nextwind;

title(_title);

xlabel(_xlabel);

ylabel(_ylabel);

xy(seqa(1,1,rows(x)),x);

endwind;



endif;

retp(resid);

endp;

/****************************************************************************/











/***************************************************************************

**                             Periodo.g

**                             ---------

****************************************************************************

**                        Estimates the Periodogram

**                        -------------------------

**                        Iww = periodo(x, rrr)

**

**            Iww = periodogram  of x at frequencies 1/2pi to rrr/2pi

***************************************************************************/



proc periodo(x,rrr);

local Iww,xbar,gam0,gam,j,t,ttt,w;

t=rows(x);

/*Estimation of autocorrelations used in periodogram*/

xbar = 1/T*sumc(x);

gam0 = 1/T*(x-xbar)'(x-xbar);

gam = zeros(T-1,1);

j = 1;

do until j>t-1;

        gam[j] =1/T*(x[1+j:T]-xbar)'(x[1:T-j]-xbar);

        j = j+1;

endo;



/*Estimation of Periodogram at first rrr values*/

Iww = zeros(minc(rrr|floor(rows(x)/2)),1);

    /*periodogram at freq's 2pi/T to r*2pi/T */

ttt = 1;

do until ttt>minc(rrr|floor(rows(x))/2);

        w = 2*pi*ttt/T;

        Iww[ttt] = 1/(2*pi)*(gam0+2*gam'cos(w*seqa(1,1,T-1)));

        ttt = ttt+1;

endo;



retp(Iww);

endp;


