library pgraph;

@***
loaddata: loads vwradtb.ssv: value weighted returns and risk free rate
input: mo1 = starting date month. (set at zero to ignore)
       yr1 = starting date year (set at zero to ignore)
       moN = ending date month (set at zero to ignore)
       yrN = ending date yr (set at zero to ignore)
       To use full sample set all inputs to zero
****@
proc(6) = loaddata(mo1,yr1,moN,yrN);
local data,yr,mo,day,rd,rnd,tbr;
local _start,_end;
load data[924,6] = /home/amaynard/share/data/stock/vwradtb.csv;
yr    = data[.,1];
mo    = data[.,2];
if mo1 == 0 and yr1 ==0;
  _start =1;
else;
  _start = finddate(mo~yr,mo1,yr1);
endif;
if moN==0 and yrN==0;
 _end = rows(data);
else;
  _end = finddate(mo~yr,moN,yrN);
endif;
data = data[_start:_end,.]; @select subsample@
yr    = data[.,1];
mo    = data[.,2];
day   = data[.,3];
rd    = data[.,4]; @nom. return with dividend@
rnd   = data[.,5]; @nom. retun excluding dividends@
tbr   = data[.,6]; @1 month treasury bill rate@
@** logical checks **@
@** may detect gross loading errors or data file corruption **@
if sumc(yr[2:rows(yr)]-yr[1:rows(yr)-1].<0) == 0;
else;
  "loaddata: error: years decreasing";
  "view: years";
  wait;
  yr;
  end;
endif;
if sumc(mo.>12+mo.<0) ==0;
else;
  "loaddata: error: month not in range";
  "view: month";
  wait;
  mo;
  end;
endif;
if sumc(day.>31+day.<0) == 0;
else;
  "loaddata: error: day not in range";
  "view: day";
  wait;
  day;
  end;
endif;
if sumc(rd.<rnd)==0;
else;
  "loaddata: error: return with dividend<return without dividend";
  "view: yr~mo~rd~rnd";
  wait;
  yr~mo~rd~rnd;
endif;
if sumc(mo[1]==mo1 + yr[1]==yr1 +mo[rows(mo)]==moN+yr[rows(yr)]==yrN-4)==0;
else;
  "loaddata: error: start and end dates don't match specified values";
  wait;
endif;
retp(yr,mo,day,rd,rnd,tbr);
endp;



@****
tplot: Plots series with dates on x axis
x - series to plot.
library pgraph must be set
****@
proc(0) = tplot(yr,mo,x);
local xx,begdate,enddate,increm,seq;
xx = yr~mo~x;
xx = packr(xx); @remove columns with missing values@
yr = xx[.,1];
mo = xx[.,2];
x = xx[.,3];
begdate = yr[1]+1/12*mo[1];
enddate = yr[rows(yr)]+1/12*mo[rows(mo)];
increm = (enddate-begdate)/(rows(yr)-1);
seq = seqa(begdate,increm,rows(yr)); @date seqence@
xtics(yr[1],yr[rows(yr)],1,12);
_pdate=0;
xy(seq,x);
endp;



/*****************
mkldpr: make log dividend price ratios
input:
       yr - N by 1 years
       mo - N by 1 months
       rd - return with dividend
       rnd - return excluding dividend
       _graph - 1 to plot dividend yield, zero to skip
output ldpr = [ldpr1,ldpr2]: 
       N by 2 matrix with some missing values at top
       ldpr1 N by 1: ln(D_t/P_t) simple log dividend price ratio
       ldpr2 N by 1: ln((D_t+D_{t-1}+...+D_{t-11})/P_t) smoothed log dividend price ratio
                     follows Campbell,Lo,MacKinlay p286
		     
*****************/

proc(1) = mkldpr(yr,mo,rd,rnd,_graph);
local pr,dpr,ldpr1,prlag,dprlag,dplagr,kk,ldpr2,ldpr,ret,ldpr3;
pr = rnd+1;       @price ratio: P_t/P_{t-1}@
dpr = (rd-rnd)./pr; @dividend price ratio: D_t/P_t@
ldpr1 = ln(dpr);   @log dividend price ratio: d_t-p_t@
prlag  = miss(zeros(rows(pr),12),0); @lags of price ratio (pr): 
         [P_{t-1}/P_{t-2},P_{t-2}/P_{t-3},..,P_{t-11}/P_{t-12}@
dprlag = miss(zeros(rows(pr),12),0); @lags of dividend price ratio (dpr): 
          [D_{t-1}/D_{t-1},D_{t-2}/P_{t-2},..,D_{t-11}/D_{t-11}@
dplagr = miss(zeros(rows(dpr),12),0); @ratio of dividend to lag prices
         [D_{t-1}/P_t,D_{t-2}/P_t,..,D_{t-11}/P_t@
kk =1;
do until kk>12; @form prlag,dprlag@
  prlag[1+kk:rows(prlag),kk] = pr[1:rows(prlag)-kk];    
  dprlag[1+kk:rows(dprlag),kk] = dpr[1:rows(dprlag)-kk];    
  kk=kk+1;
endo;

dplagr[.,1]  = dprlag[.,1]./pr; 
kk =2;
do until kk>12; @form dplagr@
dplagr[.,kk] = dprlag[.,kk]./(pr.*prodc(prlag[.,1:kk-1]'));  
  kk=kk+1;
endo;

ldpr2 = ln(dpr+sumc(dplagr[.,1:11]')); @log dividend price ratio as defined on 
      p286 of Campbell,Lo,MacKinlay: 
      ln(D_t/P_t + D_{t-1}/P_t+D_{t-2}/P_t + ... +D_{t-11}/P_t})@
ldpr3 = ln(sumc(dplagr[.,1:12]'));      @log dividend price ratio:
      alternate def. using
      ln(D_{t-1}/P_t + D_{t-1}/P_t+D_{t-2}/P_t + ... +D_{t-12}/P_t})@
      
ldpr = ldpr1~ldpr2~ldpr3;
if _graph ==1;
  title("simple log dividiend price ration");
  tplot(yr,mo,ldpr1);
  ret = tkf2ps("graphic.tkf", "/home/amaynard/share/data/stock/ldpr1.ps");
  >rm graphic.tkf; 
  "To view simple long dividend price ratio: type: gv /home/amaynard/share/data/stock/ldpr1.ps";
  title("smoothed log dividend price ratio");
  tplot(yr,mo,ldpr2);
  ret = tkf2ps("graphic.tkf","/home/amaynard/share/data/stock/ldpr2.ps");
  "To view smoothed long dividend price ratio: type: gv /home/amaynard/share/data/stock/ldpr2.ps";
  "Compare to: Campbell,Lo,MacKinlay, Fig 7.3, p284";
  >rm graphic.tkf; 
endif;  
/*
format 2,2;
@test@ "yr mo D_t/_P_{t-1}";
@test@ yr[1:12]~mo[1:12]~rd[1:12]-rnd[1:12];
@test@ "yr mo P_t/P_{t-1}";
@test@ yr[1:12]~mo[1:12]~pr[1:12]~(rnd[1:12]+1);
@test@ "yr mo D_t/P_t";
@test@ yr[1:12]~mo[1:12]~dpr[1:12];
@test@ "yr mo pr prlag9 prlag10 prlag11";
@test@ yr[1:12]~mo[1:12]~prlag[1:12,9]~prlag[1:12,10]~prlag[1:12,11];
@test@ "yr mo pr dpr0 dpr1 dpr2";
@test@ yr[1:12]~mo[1:12]~dpr[1:12]~dprlag[1:12,.];@~dprlag[1:12,2]~dprlag[1:12,3]~dprlag[1:12,4];@
@test@ "yr mo dplagr1 ";
@test@ yr[1:12]~mo[1:12]~dplagr[1:12,.];
@test@ "yr mo ldpr2 ";
@test@ yr[12]~mo[12]~ldpr2[12];
*/
retp(ldpr);
endp;


@***
mkkplrd: make k-period (nominal) log return with dividends
inputs:
  _k: column vector or return horizons or length
  dates: matrix or column of dates. For example dates = [yr,mo]
       used only for checking, printing,etc. N rows.
  rd: N by 1 column vector: net returns with dividends (not logged)	 
outputs:
  kplrd: k-period log return with dividends
         An N by rows(_k) matrix
         kplrd[t,j] = ln(rd_{t+1})+...+ln(rd_{t+_k[j]})
****@
proc(1) = mkkplrd(dates,rd,_k);
local lrd,lrdleads,y_k,kk,kplrd;
@form k- period nominal log return@
lrd = ln(1+rd); @nominal log return: r_t = ln(1+R_t): 
                see Campbell,Lo,MacKinlay, p254@
lrdleads = miss(zeros(rows(lrd),maxc(_k)),0); @cols are lrd_{k+1},lrd_{k+2},...,lrd_{t+masc(_k)@
y_k =  miss(zeros(rows(lrd),rows(_k)),0); @matrix of k period returns. Each col. corresponds
                                           to a k value in _k. entry i,j is given by
					   lrd_{t+1}+...+lrd_{t+_k_{j}}@
kk =1;
do until kk>maxc(_k);
  lrdleads[1:rows(lrd)-kk,kk] = lrd[1+kk:rows(lrd)];  
  kk=kk+1;
endo;
kk =1;
do until kk>rows(_k);
  y_k[.,kk] = sumc(lrdleads[.,1:_k[kk]]');
  kk=kk+1;
endo;
kplrd = y_k;
retp(kplrd);
endp;
@**************@



@***
mkklrr: make k-period (nominal) log real return
inputs:
  _k: column vector or return horizons or length
  lrr: N by 1 column vector: log real returns
outputs:
  kplrd: k-period log return with dividends
         An N by rows(_k) matrix
         kplrd[t,j] = ln(rd_{t+1})+...+ln(rd_{t+_k[j]})
****@
proc(1) = mkkplrr(lrr,_k);
local lrrleads,y_k,kk,kplrr;
@form k- period nominal log return@
lrrleads = miss(zeros(rows(lrr),maxc(_k)),0); @cols are lrr_{k+1},lrd_{k+2},...,lrr_{t+masc(_k)@
y_k =  miss(zeros(rows(lrr),rows(_k)),0); @matrix of k period returns. Each col. corresponds
                                           to a k value in _k. entry i,j is given by
					   lrd_{t+1}+...+lrd_{t+_k_{j}}@
kk =1;
do until kk>maxc(_k);
  lrrleads[1:rows(lrr)-kk,kk] = lrr[1+kk:rows(lrr)];  
  kk=kk+1;
endo;
kk =1;
do until kk>rows(_k);
  y_k[.,kk] = sumc(lrrleads[.,1:_k[kk]]');
  kk=kk+1;
endo;
kplrr = y_k;
retp(kplrr);
endp;
@**************@




@***
mkyk: make k-period return (y) from one period return
inputs:
  _k: column vector or return horizons or length
  y: N by 1 column vector of returns
outputs:
  kpy: k-period returns
         An N by rows(_k) matrix
         kpy[t,j] = (y_{t+1})+...+y{t+_k[j]})
****@
@**************************************@
proc(1) = mkyk(y,_k);
local yleads,y_k,kk,kpy;
/*form k- period nominal log return*/
yleads = miss(zeros(rows(y),maxc(_k)),0); /*cols are y_{k+1},lrd_{k+2},...,y_{t+masc(_k)*/
y_k =  miss(zeros(rows(y),rows(_k)),0); /*matrix of k period returns. Each col. corresponds
                                           to a k value in _k. entry i,j is given by
					   y_{t+1}+...+y_{t+_k_{j}}*/
kk =1;
do until kk>maxc(_k);
  yleads[1:rows(y)-kk,kk] = y[1+kk:rows(y)];  
  kk=kk+1;
endo;
kk =1;
do until kk>rows(_k);
  y_k[.,kk] = sumc(yleads[.,1:_k[kk]]');
  kk=kk+1;
endo;
kpy = y_k;
retp(kpy);
endp;
@***********************@
@**************@







/*********
findate:
  output: y is the row of the date your looking for in the matrix of dates called dates.
  indput: dates: is a matrix of dates with months in col1, year in column2
                         month and year: specify the date of interest
********/
	proc(1) = finddate(dates,month,year);
	local ind1, ind2, ind3, y, ind1a,ind1b,ind2a,ind2b;
	ind1a = dates[.,1] .> month - 0.1; ind1b = dates[.,1] .< month + 0.1;  
        ind1 = ind1a .* ind1b;@ones for correct month@
        ind2a = dates[.,2] .> year - .01; ind2b = dates[.,2] .< year + .01;    
	ind2 = ind2a .* ind2b;   @ones for correct year@
	ind3 = ind1.*ind2;                  @ones for correct month and year@
	y = ind3'seqa(1,1,rows(dates)); @pulls out row number of correct date@
	retp(y);
	endp;

@*****@




/********
mkkpslx: make k-period sum of lag Xs.
 input: 
    x:  N by 1 vector x[t] = x_t
    _k: vector of lag values (_k-1) is number of lags (integer)
 output:
    kpslx: N by rows(_k) matrix of k period sum oflag regressors. 
     Each col. corresponds to a k value in _k. entry i,j is given 
     by x_{t}+...+lrd_{t+1-_k_{j}}
 Note: missing values where time period falls out of data range
 ********/
proc(1) =mkkpslx(x,_k);
local xlags,kk,kpslx;
xlags = miss(zeros(rows(x),maxc(_k)),0); @cols are x{t},x{t-1},...,x{t+1-maxc(_k)}@
kk =1;
do until kk>maxc(_k);
  xlags[1+kk-1:rows(x),kk] = x[1:rows(x)-kk+1];  
  kk=kk+1;
endo;
kpslx =  miss(zeros(rows(x),rows(_k)),0);   @matrix of k period sum oflag regressors. 
     Each col. corresponds to a k value in _k. entry i,j is given by x_{t}+...+lrd_{t+1-_k_{j}}@
kk =1;
do until kk>rows(_k);
  kpslx[.,kk] = sumc(xlags[.,1:_k[kk]]');
  kk=kk+1;
endo;
retp(kpslx);
endp;
 

/******
mklp: make log price
 Input:
   dates: N by r Matrix of dates: each row is a date
   rnd:   N by 1 vector: return excluding dividends
 Output:
   lp: log price level
   Note: sets first year as base at 100
*******/
proc(1) = mklp(dates,rnd);
local p,lp,t;
p = miss(zeros(rows(rnd),1),0);
p[1] = 100;  @set base year of index@
t =2;
do until t>rows(rnd);
  p[t] = (1+rnd[t])*p[t-1]; @since 1+rnd_t = p_t/p_{t-1}@
  t =t+1;
endo;
lp = ln(p);
retp(lp);
endp;


@*******************dsim.g************************@
@dsim:  produces simulated long memory series@
/*
** simulate I(d) data
** input  = u
** output = (I-L)^-d u
*/

proc dsim(u,d);
local nn,y,b0,k,b,ur;

nn= rows(u);
b = zeros(nn,1);
y = zeros(nn,1);
b0=1;
k =1;
do until k>nn;
   if k==1;
      b[k]=d;
   else;
      b[k]=(k+d-1)*b[k-1]/k;
   endif;
   k = k+1;
endo;
b=b0|b;
k =1;
do until k>nn;
      ur  = rev(u[1:k]);
      y[k]=b[1:k]'ur;
      k = k+1;
endo;
retp(y);
endp;
